@@ -60,15 +60,18 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         }
         /* scale layout fix */
         scale trough {
-            margin-top: 7px;
-            margin-bottom: -7px;
+            margin-top: 6px;
+            margin-bottom: -6px;
         }
         '''
-        if (Gtk.get_major_version(), Gtk.get_minor_version()) >= (4, 12):
+
+        gtk_version = (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())
+
+        if gtk_version >= (4, 12, 0):
             css_provider.load_from_string(css)
         # XXX: remove workaround when merged and pygobject 3.46 is widespread
         # https://gitlab.gnome.org/GNOME/pygobject/-/merge_requests/231/
-        elif (Gtk.get_major_version(), Gtk.get_minor_version()) >= (4, 9):
+        elif gtk_version >= (4, 9, 0):
             css_provider.load_from_data(css, -1)
         else:
             css_provider.load_from_data(css.encode())
@@ -96,7 +99,7 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
 
         about_button = Gtk.Button(
             action_name='app.about',
-            icon_name='info-symbolic',
+            icon_name='open-menu-symbolic',
             has_frame=False,
             tooltip_text='About',
         )
@@ -261,8 +264,6 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         if self.frame:
             self.grid.remove(self.frame)
             self.frame = None
-            # forget old size
-            self.set_default_size(-1, -1)
 
         if self.device is None:
             return
@@ -275,7 +276,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         stack = Gtk.Stack(transition_type=Gtk.StackTransitionType.SLIDE_LEFT_RIGHT, transition_duration=500)
         stack_sw = Gtk.StackSwitcher(stack=stack, hexpand=True, halign=Gtk.Align.CENTER)
         stack_box.append(stack_sw)
-        stack_box.append(stack)
+        scrolledwindow = Gtk.ScrolledWindow(child=stack, propagate_natural_height=True, hscrollbar_policy=Gtk.PolicyType.NEVER, vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
+        stack_box.append(scrolledwindow)
 
         footer = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin_top=10)
         stack_box.append(footer)
@@ -283,10 +285,10 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
         self.frame.attach(stack_box, 0, 1, 1, 1)
         self.stack = stack
 
-        for page in self.camera.get_ctrl_pages():
+        for page_n, page in enumerate(self.camera.get_ctrl_pages()):
             page_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
             if page.target == 'main':
-                stack.add_titled(page_box, page.title.lower(), page.title)
+                stack.add_titled(page_box, str(page_n), page.title)
             elif page.target == 'footer':
                 sep = Gtk.Separator(margin_bottom=10)
                 footer.append(sep)
@@ -373,9 +375,23 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
                         refresh = Gtk.Button(icon_name='edit-undo-symbolic', valign=Gtk.Align.CENTER, halign=Gtk.Align.START, has_frame=False)
                         refresh.connect('clicked', lambda e, c=c, sc=scale: sc.get_adjustment().set_value(c.default))
                         ctrl_box.append(refresh)
-                        ctrl_box.append(scale)
+                        scale_stack = Gtk.Stack(transition_type=Gtk.StackTransitionType.SLIDE_LEFT_RIGHT, transition_duration=500)
+                        scale_box = Gtk.Box(halign=Gtk.Align.END)
+                        spin_box = Gtk.Box(halign=Gtk.Align.END)
+                        prev_button = Gtk.Button(icon_name='go-previous-symbolic', has_frame=False, opacity=0.2)
+                        prev_button.connect('clicked', lambda e, st=scale_stack, sc=scale_box: st.set_visible_child(sc))
+                        next_button = Gtk.Button(icon_name='go-next-symbolic', has_frame=False, opacity=0.2)
+                        next_button.connect('clicked', lambda e, st=scale_stack, sp=spin_box: st.set_visible_child(sp))
+                        scale_box.append(scale)
+                        scale_box.append(next_button)
+                        spin = Gtk.SpinButton(adjustment=scale.get_adjustment())
+                        spin_box.append(spin)
+                        spin_box.append(prev_button)
+                        scale_stack.add_child(scale_box)
+                        scale_stack.add_child(spin_box)
+                        ctrl_box.append(scale_stack)
                         c.gui_value_set = scale.set_value
-                        c.gui_ctrls += [scale, refresh]
+                        c.gui_ctrls += [scale, spin, refresh]
                         c.gui_default_btn = refresh
 
                     elif c.type == 'boolean':
@@ -392,18 +408,12 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
 
                     elif c.type == 'button':
                         filtered_menu = [m for m in c.menu if m.value is not None and not m.gui_hidden]
-                        children_per_line = min(len(filtered_menu), 4)
-                        box = Gtk.FlowBox(orientation=Gtk.Orientation.HORIZONTAL, homogeneous=True,
-                                        min_children_per_line=children_per_line, max_children_per_line=children_per_line,
-                                        selection_mode=Gtk.SelectionMode.NONE, hexpand=True,
-                                        # halign=Gtk.Align.END
-                                        # GTK4 workaround
-                                        halign=Gtk.Align.FILL if len(filtered_menu) > 4 else Gtk.Align.END)
-                        box.set_filter_func(lambda child: child.set_focusable(False) or True)
+                        children_per_line = 4
+                        grid = Gtk.Grid(column_homogeneous=True, row_homogeneous=True, column_spacing=4, row_spacing=4, hexpand=True, halign=Gtk.Align.END)
                         refresh = Gtk.Button(icon_name='edit-undo-symbolic', valign=Gtk.Align.CENTER, halign=Gtk.Align.START, has_frame=False)
                         ctrl_box.append(refresh)
-                        ctrl_box.append(box)
-                        for m in filtered_menu:
+                        ctrl_box.append(grid)
+                        for idx, m in enumerate(filtered_menu):
                             b = Gtk.Button(label=m.name, valign=Gtk.Align.CENTER)
                             b.connect('clicked', lambda e, c=c, m=m: self.update_ctrl(c, m.text_id))
                             if c.child_tooltip:
@@ -415,11 +425,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
                                     self.update_ctrl(c, m.lp_text_id)
                                 ])
                                 b.add_controller(lp)
-                            if m.use_shortcut:
-                                b.set_label('_' + m.name)
-                                b.set_use_underline(True)
-                            box.append(b)
-                            c.gui_ctrls += b
+                            grid.attach(b, idx % children_per_line, idx // children_per_line, 1, 1)
+                            c.gui_ctrls += [b]
                         if c.default is not None:
                             refresh.connect('clicked', lambda e,c=c: self.update_ctrl(c, c.default))
                         c.gui_ctrls += [refresh]
@@ -478,6 +485,8 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
                             c.gui_default_btn = refresh
 
         self.update_ctrls_state()
+        _, natsize = self.get_preferred_size()
+        self.set_default_size(natsize.width, natsize.height)
 
     def close_notify(self):
         self._revealer.set_reveal_child(False)
@@ -515,9 +524,9 @@ class CameraCtrlsWindow(Gtk.ApplicationWindow):
 
     def update_ctrl_state(self, c):
         for gui_ctrl in c.gui_ctrls:
-            gui_ctrl.set_sensitive(not c.inactive)
+            gui_ctrl.set_sensitive(not c.inactive and not c.readonly)
         if c.gui_default_btn is not None:
-            visible = not c.inactive and (
+            visible = not c.inactive and not c.readonly and (
                 c.default is not None and c.value is not None and c.default != c.value or \
                 c.get_default is not None and not c.get_default()
             )
@@ -663,7 +672,18 @@ class CameraCtrlsApp(Gtk.Application):
         action.connect('activate', self.on_quit)
         self.add_action(action)
 
-        self.set_accels_for_action('app.quit',["<Primary>Q"])
+        action = Gio.SimpleAction.new('alt_n', GLib.VariantType('i'))
+        action.connect('activate', self.on_alt_n)
+        self.add_action(action)
+
+        self.set_accels_for_action('app.quit', ["<Primary>Q"])
+        self.set_accels_for_action('app.alt_n(0)', ["<Alt>1"])
+        self.set_accels_for_action('app.alt_n(1)', ["<Alt>2"])
+        self.set_accels_for_action('app.alt_n(2)', ["<Alt>3"])
+        self.set_accels_for_action('app.alt_n(3)', ["<Alt>4"])
+        self.set_accels_for_action('app.alt_n(4)', ["<Alt>5"])
+        self.set_accels_for_action('app.alt_n(5)', ["<Alt>6"])
+        self.set_accels_for_action('app.alt_n(6)', ["<Alt>7"])
 
     def do_activate(self):
         self.window = CameraCtrlsWindow(application=self, title='Cameractrls')
@@ -675,7 +695,7 @@ class CameraCtrlsApp(Gtk.Application):
         about.set_authors(['Gergo Koteles <soyer@irl.hu>'])
         about.set_artists(['Jorge Toledo https://lacajita.es'])
         about.set_copyright('Copyright © 2022 - 2024 Gergo Koteles')
-        about.set_license_type(Gtk.License.GPL_3_0)
+        about.set_license_type(Gtk.License.LGPL_3_0)
         about.set_logo_icon_name('hu.irl.cameractrls')
         about.set_website(ghurl)
         about.set_website_label('GitHub')
@@ -685,11 +705,16 @@ class CameraCtrlsApp(Gtk.Application):
     def on_quit(self, action, param):
         self.window.close()
 
+    def on_alt_n(self, action, param):
+        page_n = str(param)
+        if self.window.stack and self.window.stack.get_child_by_name(page_n):
+            self.window.stack.set_visible_child_name(page_n)
+
     def check_preview_open(self, p):
         # if process returned
         if p.poll() is not None:
             (stdout, stderr) = p.communicate()
-            errstr = str(stderr, 'utf-8')
+            errstr = stderr.decode()
             sys.stderr.write(errstr)
             if p.returncode != 0:
                 self.window.notify(errstr.strip())
